/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#include "rm_scan.h"
#include "rm_file_handle.h"

/**
 * @brief 初始化file_handle和rid
 * @param file_handle
 */
RmScan::RmScan(const RmFileHandle *file_handle) : file_handle_(file_handle) {
    // Todo:
    // 初始化file_handle和rid（指向第一个存放了记录的位置）
    rid_.page_no = RM_FIRST_RECORD_PAGE; //不初始化为-1，方便for循环不执行时直接赋值，后续判断为is_end
    rid_.slot_no = -1;
    // num_pages永远在递增，不会减少，所以for中判断条件没问题
    for(int i = RM_FIRST_RECORD_PAGE; i <= file_handle->file_hdr_.num_pages - 1; i++)
    {
        RmPageHandle page_handle = file_handle_->fetch_page_handle(i);
        rid_.page_no = i;
        if(page_handle.page_hdr->num_records > 0)
        {   
            rid_.slot_no = Bitmap::first_bit(true, 
                                             page_handle.bitmap, 
                                             file_handle_->file_hdr_.num_records_per_page);
            file_handle_->buffer_pool_manager_->unpin_page(page_handle.page->get_page_id(), false); 
            break;
        }
    }
}

/**
 * @brief 找到文件中下一个存放了记录的位置
 */
void RmScan::next() {
    // Todo:
    // 找到文件中下一个存放了记录的非空闲位置，用rid_来指向这个位置
    // 调用时在for循环中判断是否到达末尾，该函数内部不进行判断!
    // 未到达文件末尾，判断是否在当前页末尾
    for(int i = rid_.page_no; i < file_handle_->file_hdr_.num_pages; /*i在else中++递增*/)
    {
        RmPageHandle page_handle = file_handle_->fetch_page_handle(i);// pages单纯删记录之后可能不连续
        rid_.slot_no = Bitmap::next_bit(true, 
                                        page_handle.bitmap, 
                                        file_handle_->file_hdr_.num_records_per_page,
                                        rid_.slot_no);
        // 空白页会进入else中继续下一页，直到退出for循环
        // 不是空白页就读此页中的记录
        if(rid_.slot_no != file_handle_->file_hdr_.num_records_per_page)
        {
            // 正常情况直接break就行了，slot_no已经在上边被赋过值了
            file_handle_->buffer_pool_manager_->unpin_page(page_handle.page->get_page_id(), false);                                
            break;
        }
        else
        {
            // 当前页记录已经遍历完毕，继续遍历下一页
            rid_.page_no = ++i;
            rid_.slot_no = -1;
            file_handle_->buffer_pool_manager_->unpin_page(page_handle.page->get_page_id(), false); 
            continue;
        }
    }
}

/**
 * @brief ​ 判断是否到达文件末尾
 */
bool RmScan::is_end() const {
    // Todo: 修改返回值
    if(rid_.page_no >= file_handle_->file_hdr_.num_pages - 1 && rid_.slot_no == -1)
    {
        return true;
    }
    else
    {
        return false;
    }
}

/**
 * @brief RmScan内部存放的rid
 */
Rid RmScan::rid() const {
    return rid_;
}